from __future__ import annotations

import logging
from abc import ABC
from typing import Literal, Union

from pydantic import Field
from soda_core.common.logging_constants import soda_logger
from soda_core.model.data_source.data_source import DataSourceBase
from soda_sqlserver.common.data_sources.sqlserver_data_source_connection import (
    SqlServerActiveDirectoryAuthentication,
    SqlServerActiveDirectoryInteractiveAuthentication,
    SqlServerActiveDirectoryPasswordAuthentication,
    SqlServerActiveDirectoryServicePrincipalAuthentication,
    SqlServerConnectionProperties,
    SqlServerDataSourceConnection,
    SqlServerPasswordAuth,
)

logger: logging.Logger = soda_logger


CONTEXT_AUTHENTICATION_DESCRIPTION = "Use context authentication"


# All of these classes are just copies of the SQLServerConnectionProperties classes, but with the Synapse type
class SynapseConnectionProperties(SqlServerConnectionProperties, ABC):
    pass


class SynapsePasswordAuth(SqlServerPasswordAuth, SynapseConnectionProperties):
    pass


class SynapseActiveDirectoryAuthentication(SqlServerActiveDirectoryAuthentication, SynapseConnectionProperties):
    pass


class SynapseActiveDirectoryInteractiveAuthentication(
    SqlServerActiveDirectoryInteractiveAuthentication, SynapseConnectionProperties
):
    pass


class SynapseActiveDirectoryPasswordAuthentication(
    SqlServerActiveDirectoryPasswordAuthentication, SynapseConnectionProperties
):
    pass


class SynapseActiveDirectoryServicePrincipalAuthentication(
    SqlServerActiveDirectoryServicePrincipalAuthentication, SynapseConnectionProperties
):
    pass


class SynapseDataSource(DataSourceBase, ABC):
    type: Literal["synapse"] = Field("synapse")

    connection_properties: Union[
        SynapsePasswordAuth,
        SynapseActiveDirectoryInteractiveAuthentication,
        SynapseActiveDirectoryPasswordAuthentication,
        SynapseActiveDirectoryServicePrincipalAuthentication,
    ] = Field(..., alias="connection", description="Synapse connection configuration")


class SynapseDataSourceConnection(SqlServerDataSourceConnection):
    def _get_autocommit_setting(self) -> bool:
        return True  # Synapse requires autocommit to be True.



from __future__ import annotations

import os
from typing import Optional

from soda_sqlserver.test_helpers.sqlserver_data_source_test_helper import (
    SqlServerDataSourceTestHelper,
)


class SynapseDataSourceTestHelper(SqlServerDataSourceTestHelper):
    def _create_database_name(self) -> Optional[str]:
        return os.getenv("SYNAPSE_DATABASE", "sodatestingsynapse")

    def _create_data_source_yaml_str(self) -> str:
        """
        Called in _create_data_source_impl to initialized self.data_source_impl
        self.database_name and self.schema_name are available if appropriate for the data source type
        """
        return f"""
            type: synapse
            name: {self.name}
            connection:
                host: '{os.getenv("SYNAPSE_HOST", "localhost")}'
                port: '{os.getenv("SYNAPSE_PORT", "1433")}'
                database: '{os.getenv("SYNAPSE_DATABASE", "sodatestingsynapse")}'
                authentication: '{os.getenv("SYNAPSE_AUTHENTICATION_TYPE", "activedirectoryserviceprincipal")}'
                client_id: '{os.getenv("SYNAPSE_CLIENT_ID")}'
                client_secret: '{os.getenv("SYNAPSE_CLIENT_SECRET")}'
                trust_server_certificate: true
                driver: '{os.getenv("SYNAPSE_DRIVER", "ODBC Driver 18 for SQL Server")}'
        """



#!/usr/bin/env python

from setuptools import setup

package_name = "soda-synapse"
package_version = "4.0.5b1"
description = "Soda Synapse V4"

requires = [
    f"soda-core=={package_version}",
    f"soda-sqlserver=={package_version}",
]

setup(
    name=package_name,
    version=package_version,
    install_requires=requires,
    package_dir={"": "src"},
    entry_points={
        "soda.plugins.data_source.synapse": [
            "SynapseDataSourceImpl = soda_synapse.common.data_sources.synapse_data_source:SynapseDataSourceImpl",
        ],
    },
)
